/* mapview - map viewer for PalmOS
   Copyright (C) 2005  Edwin Steiner <edwin.steiner@gmx.net>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <PalmOS.h>
#include <string.h>
#include "memory.h"
#include "rsrc.h"
#include "debug.h"
#include "mapfile.h"
#include "map.h"
#include "mapview.h"

/* SiEd headers */
#include "filedialog.h"
#include "utility.h"
#include "file_interface.h"

extern "C" {

int
mapview_filedialog_init(void)
{
        SiUtility::check_vfs_support();
        SiFileDialog::initialise();
        return 1;
}

int
mapview_filedialog_cleanup(void)
{
        SiFileDialog::exit();
        return 1;
}

bool
mapview_filedialog_open(void)
{
        SiFile *file;

        file = SiFileDialog::ShowOpenFileDialog(NULL,NON_VFS);
        if (!file)
                return false;

        if (file->get_vol_ref() == NON_VFS) {
                LocalID dbid;
                UInt16 cardno;

                /* a map in internal memory */
                cardno = file->get_cardno();
                dbid = file->get_dbid();
                delete file;
                return mapview_open_map(cardno,dbid);
        }
        else {
                mapview_mapfile_t *mapfile;

                /* a map in an VFS file */
                mapfile = mapview_mapfile_open(file->get_vol_ref(),file->get_location());
                if (!mapfile) {
                        mapview_message_fmt("Could not open mapfile: %s",file->get_location());
                        delete file;
                        return false;
                }
                else {
                        delete file;
                        return mapview_open_map_mapfile(mapfile);
                }
        }
}

} /* extern "C" */

/* vim: et sts=8 sw=8 ai
 */
