/* mapview - map viewer for PalmOS
   Copyright (C) 2005  Edwin Steiner <edwin.steiner@gmx.net>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef _MAPFILE_H_
#define _MAPFILE_H_

#include <VFSMgr.h>

/* these structs define the mapfile format */
/* see MapfileFormat.txt for details       */

#define MAPVIEW_MAGIC_ANY         0

#define MAPVIEW_MAGIC_HEADER      0x4d560100U
#define MAPVIEW_MAGIC_MAPHEADER   0x4d560200U
#define MAPVIEW_MAGIC_LAYERHEADER 0x4d560300U
#define MAPVIEW_MAGIC_TILEDIR     0x4d560400U
#define MAPVIEW_MAGIC_TILE        0x4d560500U

typedef struct mapview_mapfile_t mapview_mapfile_t;

typedef struct mapview_mapfile_chunk_t mapview_mapfile_chunk_t;
typedef struct mapview_mapfile_header_t mapview_mapfile_header_t;
typedef struct mapview_mapfile_mapheader_t mapview_mapfile_mapheader_t;
typedef struct mapview_mapfile_layerheader_t mapview_mapfile_layerheader_t;
typedef struct mapview_mapfile_tiledir_t mapview_mapfile_tiledir_t;
typedef struct mapview_mapfile_tile_t mapview_mapfile_tile_t;

struct mapview_mapfile_chunk_t {
        UInt32  magic;
        UInt32  size;
};

struct mapview_mapfile_header_t {
        mapview_mapfile_chunk_t chunk;

        UInt16  version;
};

struct mapview_mapfile_mapheader_t {
        mapview_mapfile_chunk_t chunk;

        UInt32  mapsizex;
        UInt32  mapsizey;
        UInt16  nlayers;
        UInt16  density;
        UInt16  bpp;
        UInt16  reserved;

        UInt32  layerofs[0];
};

struct mapview_mapfile_layerheader_t {
        mapview_mapfile_chunk_t chunk;

        UInt16  scale;
        UInt16  tilesizex;
        UInt16  tilesizey;
        UInt16  tilesx;
        UInt16  tilesy;
        UInt32  tiledirofs;
};

struct mapview_mapfile_tiledir_t {
        mapview_mapfile_chunk_t chunk;

        UInt32  tileofs[0];
};

struct mapview_mapfile_tile_t {
        mapview_mapfile_chunk_t chunk;

        UInt8   data[0];
};

/* mapfile type, consider this opaque! */

struct mapview_mapfile_t {
        FileRef  fileref;
};

/* functions for accessing mapfiles */

#ifdef __cplusplus
extern "C" {
#endif
        
mapview_mapfile_t * mapview_mapfile_open(UInt16 vol,const char *path);
void mapview_mapfile_free(mapview_mapfile_t *mapfile);

mapview_mapfile_chunk_t * mapview_mapfile_read_chunk(mapview_mapfile_t *mapfile,UInt32 ofs,UInt32 magic);
void mapview_mapfile_free_chunk(mapview_mapfile_chunk_t *chunk);

#ifdef __cplusplus
}
#endif

#endif

/* vim: et sts=8 sw=8 ai
 */
